//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
texture g_offscreenTexture;
float g_time;  


//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
sampler offscreenTextureSampler = 
sampler_state
{
    Texture = <g_offscreenTexture>;
    AddressU = BORDER;
    AddressV = BORDER;

    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position   : POSITION;   // vertex position 
    float2 Left       : TEXCOORD0;
    float2 Right      : TEXCOORD1;
    float2 Top        : TEXCOORD2;
    float2 Bottom     : TEXCOORD3;
};


//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_OUTPUT RenderSceneVS( float4 vPos : POSITION, float2 UVs : TEXCOORD0 )
{
    VS_OUTPUT Output;

	float dist = 1.0/1024.0;

	UVs.x = UVs.x + dist * 0.5;
	UVs.y = UVs.y + dist * 0.5;

	dist*=1;

	Output.Position = vPos;
	Output.Left = UVs;
	Output.Left.x = UVs.x - dist;
	
	Output.Right = UVs;
	Output.Right.x = UVs.x + dist;
	
	Output.Top = UVs;
	Output.Top.y = UVs.y + dist;
	
	Output.Bottom = UVs;
	Output.Bottom.y = UVs.y - dist;
	
    return Output;    
}


//--------------------------------------------------------------------------------------
// Pixel shader output structure
//--------------------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;  // Pixel color    
};


//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
// color with diffuse material color
//--------------------------------------------------------------------------------------
PS_OUTPUT RenderScenePS( VS_OUTPUT In ) 
{ 
	PS_OUTPUT Output;

	// Lookup mesh texture and modulate it with diffuse
	Output.RGBColor = tex2D(offscreenTextureSampler, In.Left);
	Output.RGBColor += tex2D(offscreenTextureSampler, In.Right);
	Output.RGBColor += tex2D(offscreenTextureSampler, In.Top);
	Output.RGBColor += tex2D(offscreenTextureSampler, In.Bottom);
	Output.RGBColor *= .255;

	return Output;
}


//--------------------------------------------------------------------------------------
// Renders scene 
//--------------------------------------------------------------------------------------
technique RenderScene
{
    pass P0
    {          
        VertexShader = compile vs_2_0 RenderSceneVS();
        PixelShader  = compile ps_2_0 RenderScenePS(); 
    }
}
