###############################################################################
# Main game classes
class GameObject
	# Constants
	attr_accessor :state
	Dead = 0
	Alive = 1

	# Types
	attr_accessor :type
	PlayerShip = 0
	EnemyShip = 1
	Planet = 2

	attr_accessor :tile, :vx, :vy, :size, :hp, :mass
end

class Planet < GameObject
	def x 
		@x 
	end
	def x=( in_x )
		@x = in_x
		@tile.x = in_x
	end
	def y 
		@y 
	end
	def y=( in_y )
		@y = in_y
		@tile.y = in_y
		@tile.layer = -Integer((in_y+SpaceCute::BoardHeight)*100)
	end
	
	def initialize( in_x, in_y )
		@state = Dead
		@mass = SpaceCute::PlanetaryMass
		@type = GameObject::Planet
		@hp = 500
		@tile = Scene.new_tile( "res/planet.png" ).set_scale(2,2)
		self.x= in_x
		self.y= in_y
		@vx = 0
		@vy = 0
		@size = 1.4
	end
	
	def hit
	end
	
	def update
	end
end

class Ship < GameObject
	
	def x 
		@x 
	end
	def x=( in_x )
		@smoke.x = in_x-0.6
		@x = in_x
		@tile.x = in_x
		@hearts[0].x = in_x-0.6
		@hearts[1].x = in_x-0.4
		@hearts[2].x = in_x-0.1
		@hearts[3].x = in_x+0.2
		@hearts[4].x = in_x+0.4
	end
	def y 
		@y 
	end
	def y=( in_y )
		@smoke.y = in_y
		@y = in_y
		@tile.y = in_y + 0.2
		@tile.layer = -Integer((in_y+SpaceCute::BoardHeight)*100)
		@hearts[0].y = in_y+1.1
		@hearts[1].y = in_y+1.325
		@hearts[2].y = in_y+1.4
		@hearts[3].y = in_y+1.325
		@hearts[4].y = in_y+1.1
	end
	
	def initialize( in_x, in_y )
		@smoke = PSystem.new( 20, "res/smoke.png", in_x, in_y )
		@state = Alive
		@mass = 1
		@type = GameObject::PlayerShip
		@hp = 5
		@hearts = []
		@hearts << Scene.new_tile( "res/healthheart.png" ).set_scale(0.15, 0.15).set_rotation( 50 )
		@hearts << Scene.new_tile( "res/healthheart.png" ).set_scale(0.15, 0.15).set_rotation( 20 )
		@hearts << Scene.new_tile( "res/healthheart.png" ).set_scale(0.15, 0.15).set_rotation( 0 )
		@hearts << Scene.new_tile( "res/healthheart.png" ).set_scale(0.15, 0.15).set_rotation( -20 )
		@hearts << Scene.new_tile( "res/healthheart.png" ).set_scale(0.15, 0.15).set_rotation( -50 )
		@tile = Scene.new_tile( "res/beetleship.png" )
		self.x= in_x
		self.y= in_y
		@vx = 0
		@vy = 0
		@size = 0.6
	end
	
	def hit
		@hp = @hp - 1
		if( @hp == 0 )
			@state = Dead
			@smoke.activate
			$sad_sound.play
			@mass = SpaceCute::WreckageMass
		end
		5.times do |i|
			@hearts[i].active = (i < @hp) # Heart-o-meter
		end
	end
	
	def update
		if( @hp < 1 )
			@smoke.iterate
		end
	end
end

class Particle
	attr_accessor :age, :tile, :vx, :vy, :vr
	def initialize( in_image, in_x, in_y )
		@tile = Scene.new_tile( in_image )
		@tile.blend_mode = 1
		@tile.layer = 10000
		@tile.active = false
		reset( in_x, in_y )
	end
	
	def reset( in_x, in_y )
		# reset
		@age = 0
		@vx = (rand - 0.5)/10
		@vy = (0.75+rand)/12
		@vr = (rand - 0.5)
		@tile.x = in_x
		@tile.y = in_y
		scale = 0.2 + rand*0.5
		@tile.set_scale( scale, scale )
	end
end

class PSystem
	attr_accessor :x, :y
	
	def initialize( in_particle_count, in_image, in_x = 0, in_y = 0 )
		self.x = in_x
		self.y = in_y
		@particles = []
		in_particle_count.times do
			the_particle = Particle.new( in_image, in_x, in_y )
			@particles << the_particle
		end
	end
	def iterate
		@particles.each do |particle|
			particle.age = particle.age + rand / 10
			particle.tile.x = particle.tile.x + particle.vx
			particle.tile.y = particle.tile.y + particle.vy
			particle.tile.rot = particle.tile.rot + particle.vr
			particle.tile.a = 1-particle.age
			if( particle.age > 1 )
				particle.reset( @x, @y )
			end
		end
	end
	def activate
		@particles.each do |part| 
			part.tile.active = true
			part.tile.x = @x
			part.tile.y = @y
		end
	end
end


###############################################################################
# Helpers
class Array
	def each_unique_pair
		self.each_with_index{ |a,i|
			self[(i+1)..-1].each{ |b| yield a,b }
		}
	end
end

class Vector
	attr_accessor :x, :y
	def initialize( in_x = 0, in_y = 0 )
		@x = in_x
		@y = in_y
	end
end


###############################################################################
# Sounds
$done_sound  = Scene.new_sound( "res/done.ogg"  )
$vroom_sound = Scene.new_sound( "res/vroom.ogg" )
$sad_sound   = Scene.new_sound( "res/sad.ogg"   )
$bump_sound  = Scene.new_sound( "res/bump.ogg"  )