//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
float    desaturationFactor;        // Can be used to fade to grayscale
float    highpassFreq;              // Special highpass color effect

texture  g_MeshTexture;             // Color texture for mesh
float4   g_diffuseColor;            // Tile's diffuse color

float4x4 g_mWorld;                  // World matrix for object
float4x4 g_mWorldViewProjection;    // World * View * Projection matrix


//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
sampler MeshTextureSampler = 
sampler_state
{
    Texture = <g_MeshTexture>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = CLAMP;
    AddressV = CLAMP;
};


//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position   : POSITION;   // vertex position 

	// Going to have to bring texture coords in eventually.
	float2 TextureUV  : TEXCOORD0;  // vertex texture coords 
};


//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_OUTPUT RenderSceneVS( float4 vPos : POSITION, float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;
    
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(vPos, g_mWorldViewProjection);

    // copy the texture coordinate through
	Output.TextureUV = vTexCoord0; 
	
    return Output;    
}


//--------------------------------------------------------------------------------------
// Pixel shader output structure
//--------------------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;  // Pixel color    
};


//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
// color with diffuse material color
//--------------------------------------------------------------------------------------
PS_OUTPUT RenderScenePS( VS_OUTPUT In ) 
{ 
	PS_OUTPUT Output;

	// Lookup mesh texture and modulate it with diffuse
	Output.RGBColor = tex2D(MeshTextureSampler, In.TextureUV) * g_diffuseColor;
	//Output.RGBColor.a *= 0.5;

	float grayscale = dot(float3(0.222, 0.707, 0.571), Output.RGBColor);

	// Saturation highpass filter
	if( grayscale < highpassFreq)
	{
		grayscale=0;
	}
	
	Output.RGBColor.r = lerp( Output.RGBColor.r, grayscale, desaturationFactor );
	Output.RGBColor.g = lerp( Output.RGBColor.g, grayscale, desaturationFactor );
	Output.RGBColor.b = lerp( Output.RGBColor.b, grayscale, desaturationFactor );
	return Output;
}


//--------------------------------------------------------------------------------------
// Renders scene 
//--------------------------------------------------------------------------------------
technique RenderScene
{
    pass P0
    {          
        VertexShader = compile vs_1_1 RenderSceneVS();
        PixelShader  = compile ps_2_0 RenderScenePS(); 
    }
}
