########################################################
# SpaceCute v0.3 - By: Harold.
#
# Greetz to Danc, and anyone else foolish enough to
# open this file.
#
# Fast and furious hacking, executed with impunity, 
# makes unmanageable code. It calls out into the night 
# for a rewrite... But I hear the first rule of indie
# game development is, "Don't quit your day job."
#
# However, this succeeds as a prototype by proving that 
# this concept is fun.
#
# see variables.rb for values of balancing factors
########################################################

require './variables.rb'
require './SC.rb'

SWaitingForInput = 0
SDragging        = 1
SEvaluating      = 2
SAI              = 3

$state = SWaitingForInput
$selected_ship = nil

# Background and stars
Scene.new_tile( "res/bg.png" ).set_scale( 10.24, 7.68 ).set_layer( -10000 )
SpaceCute::StarCount.times do
	x = rand * 16 - 8
	y = rand * 10 - 5
	size = 0.1+(rand*1.5)/2
	Scene.new_tile( "res/star.png" ).set_pos(x,y).set_scale(size,size).set_layer(-9900)
end

# Walls
Scene.new_tile( "res/white.png" ).set_pos( SpaceCute::BoardWidth, 0 ).set_scale( 0.05, SpaceCute::BoardHeight ).set_color( 0.5, 0.5, 0.5, 1 ).set_layer(-9800)
Scene.new_tile( "res/white.png" ).set_pos( -SpaceCute::BoardWidth, 0 ).set_scale( 0.05, SpaceCute::BoardHeight ).set_color( 0.5, 0.5, 0.5, 1 ).set_layer(-9800)
Scene.new_tile( "res/white.png" ).set_pos( 0, SpaceCute::BoardHeight ).set_scale( SpaceCute::BoardWidth, 0.05 ).set_color( 0.5, 0.5, 0.5, 1 ).set_layer(-9800)
Scene.new_tile( "res/white.png" ).set_pos( 0, -SpaceCute::BoardHeight ).set_scale( SpaceCute::BoardWidth, 0.05 ).set_color( 0.5, 0.5, 0.5, 1 ).set_layer(-9800)

# Dotted line and drag target
$dotted_line = Scene.new_tile( "res/dottedline.png" )
$dotted_line.scale_x = 0.0625
$dotted_line.active = false
$target = Scene.new_tile( "res/target.png" ).set_scale(0.3,0.3).set_color(1,1,1,0.6)
$target.active = false

# Collection of all game objects
$objs = []
$objs << Ship.new( 2, 2 )
$objs << Ship.new( 2, -4 )
$objs << Ship.new( -2, -2 )
$objs << Planet.new( -2, 2 )
$objs << Planet.new( 3, -2 )
$hurt = []

def update
	$objs.each { |obj| obj.update }
	case( $state )
	when SDragging
		mx = Scene.mouse_x
		my = Scene.mouse_y
		
		dx = mx-$selected_ship.x
		dy = my-$selected_ship.y
		c = Math.sqrt( (dx*dx)+(dy*dy) )
		theta = Math.asin( dx/c )
		$dotted_line.x = $selected_ship.x
		$dotted_line.y = $selected_ship.y
		$dotted_line.rot = -(theta / (Math::PI/180))
		if dy < 0 
			$dotted_line.rot = -$dotted_line.rot + 180
		end
	
		the_power = 0
		if (c < SpaceCute::MaxUIDrag)
			the_power = c/SpaceCute::MaxUIDrag
			$dotted_line.scale_y = c
			$target.x = $selected_ship.x+dx
			$target.y = $selected_ship.y+dy
		else
			the_power = 1
			$dotted_line.scale_y = SpaceCute::MaxUIDrag
			the_overshoot = c/SpaceCute::MaxUIDrag
			$target.x = $selected_ship.x+(dx/the_overshoot)
			$target.y = $selected_ship.y+(dy/the_overshoot)
		end
		
		$selected_ship.vx = (dx/c)*the_power
		$selected_ship.vy = (dy/c)*the_power
	when SEvaluating
		moved_something = false
		$objs.each do |obj|
			if ( obj.vx != 0 || obj.vy != 0 && obj.tile.active)
				moved_something = true
				obj.x = obj.x + (obj.vx*SpaceCute::PlayerSpeed)
				obj.y = obj.y + (obj.vy*SpaceCute::PlayerSpeed)
				obj.vx = obj.vx * SpaceCute::Friction
				obj.vy = obj.vy * SpaceCute::Friction
				obj.vx = 0 if obj.vx.abs < 0.005
				obj.vy = 0 if obj.vy.abs < 0.005
				if obj.x < -SpaceCute::BoardWidth
					obj.x = -SpaceCute::BoardWidth
					obj.vx = -obj.vx
				elsif obj.x > SpaceCute::BoardWidth
					obj.x = SpaceCute::BoardWidth
					obj.vx = -obj.vx
				end
				if obj.y < -SpaceCute::BoardHeight
					obj.y = -SpaceCute::BoardHeight
					obj.vy = -obj.vy
				elsif obj.y > SpaceCute::BoardHeight
					obj.y = SpaceCute::BoardHeight
					obj.vy = -obj.vy
				end
			end
		end
		
		$objs.each_unique_pair do |a,b|
			dist = Math.sqrt( (a.x-b.x)**2 + (a.y-b.y)**2 )
			if( dist < a.size+b.size && a.tile.active && b.tile.active)
				$bump_sound.play
				if ( $selected_ship != a && !($hurt.include?(a)) )
					a.hit
					$hurt << a
				end
				if ( $selected_ship != b && !($hurt.include?(b)) )
					b.hit
					$hurt << b
				end
						
				# Physics!
				the_collision_normal = Vector.new
				the_collision_normal.x = a.x - b.x
				the_collision_normal.y = a.y - b.y
				a_len = Math.sqrt((a.vx*a.vx)+(a.vy*a.vy))
				b_len = Math.sqrt((b.vx*b.vx)+(b.vy*b.vy))
				speed = (a_len+b_len)/SpaceCute::CollisionDivisor
				if( $selected_ship == a )
					a.vx = (a.vx + ( the_collision_normal.x * speed )) * SpaceCute::SelectedShipBounce
					a.vy = (a.vy + ( the_collision_normal.y * speed )) * SpaceCute::SelectedShipBounce
				else
					a.vx = a.vx + ( the_collision_normal.x * speed )
					a.vy = a.vy + ( the_collision_normal.y * speed )
				end
				a.vx /= a.mass
				a.vy /= a.mass
				
				if( $selected_ship == b )
					b.vx = (b.vx + -( the_collision_normal.x * speed )) * SpaceCute::SelectedShipBounce
					b.vy = (b.vy + -( the_collision_normal.y * speed )) * SpaceCute::SelectedShipBounce
				else
					b.vx = b.vx + -( the_collision_normal.x * speed )
					b.vy = b.vy + -( the_collision_normal.y * speed )
				end
				b.vx /= b.mass
				b.vy /= b.mass
			end
		end
		
		# Done moving
		unless moved_something
			$hurt = []
			$done_sound.play
			$selected_ship = nil
			$state = SWaitingForInput 
		end
	else
	end
end

def left_mouse_down
	mx = Scene.mouse_x
	my = Scene.mouse_y
	case( $state )
	when SWaitingForInput
		lowest_dist = 1984
		$objs.each do |obj|
			dist = Math.sqrt( (obj.x-mx)**2 + (obj.y-my)**2 )
			if($selected_ship.nil? || (dist<lowest_dist)) && (obj.state == Ship::Alive) && (obj.type == GameObject::PlayerShip)
				lowest_dist = dist
				$selected_ship = obj
			end
		end
		$state = SDragging
		$dotted_line.active = true
		$target.active = true
		$selected_ship.tile.a = SpaceCute::SelectedAlphaFactor
	else
	end
end

def left_mouse_up
	case( $state )
	when SDragging
		$vroom_sound.play
		$state = SEvaluating
		$selected_ship.tile.a = 1
		$dotted_line.active = false
		$target.active = false
	else
	end
end
